// src/backend/sgpemv2/resource_policies_gatekeeper.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef RESOURCE_POLICIES_GATEKEEPER_HH
#define RESOURCE_POLICIES_GATEKEEPER_HH 1

namespace sgpem
{
  class ResourcePolicyManager;
  class ResourcePolicy;
  class History;
}

#include <sgpemv2/sgpemv2-visibility.hh>

#include <vector>
#include <map>
#include <stdexcept>

#include <sgpemv2/templates/singleton.hh>

namespace sgpem
{
  class ResourcePoliciesGatekeeper;

  /** \brief FIXME document me


   */

  class SG_DLLEXPORT ResourcePoliciesGatekeeper : public Singleton<ResourcePoliciesGatekeeper>
  {
    friend class Singleton<ResourcePoliciesGatekeeper>;

  public:
    typedef ResourcePolicyManager Manager;
    typedef std::vector<Manager*> Managers;

    /**
     *  \brief Returns registered resources policy managers.
     *
     *  \return Registered resources policy managers.
     */
    const Managers&   get_registered() const;

    /**
     * \brief Register a new resources policy manager.
     * A no-op if a manager of the same type already exists.
     */
    void       register_manager(ResourcePolicyManager* manager);
    
    /**
     * \brief Unregister a given manager. If not present, it is a no-op.
     * When unregistering a manager, care should be taken to ensure that
     * the currently selected policy wasn't managed by it.
     * If so, the policy should be deactivated before removal, and current_policy reset.
     *
     * \param manager The resource policy manager to remove.
     */
    void       unregister_manager(ResourcePolicyManager* manager);

    /**
     * \brief Returns the currently active policy.
     * If no policy was previously activated for the attached ::History,
     * throws an appropriate exception.
     *
     * \return The current policy.
     */
    ResourcePolicy&   get_current_policy(History* history) throw(std::runtime_error);
    /**
      \brief Given a policy, sets it as the history current one.
      Also deactivates the old policy if present, and then activates
      the new policy before use. If old_policy == new_policy, it's a no-op.
     */
    void       activate_policy(History* history, ResourcePolicy* policy);

  private:
    /**
      \brief Private constructor.
      */
    ResourcePoliciesGatekeeper();     //private constructor.
    /**
      \brief Private copy constructor.
      */
    ResourcePoliciesGatekeeper(const ResourcePoliciesGatekeeper&);
    /**
      \brief Private assignment operator.
      */
    ResourcePoliciesGatekeeper& operator=(const ResourcePoliciesGatekeeper&);

    void deactivate_policies(const ResourcePolicyManager& manager);
    /**
     \brief Container of registered managers.
     */
    Managers                                _registered;
    /**
     \brief Container of active policies.
     */
    std::map<History*, ResourcePolicy*>     _active_policies;
  };

}//~ namespace sgpem

#endif //RESOURCE_POLICIES_GATEKEEPER_HH

